#include "TeleMod.h"

#include "SGTL5000.h"

// Define the Microphone characteristic
//
#define NUM_MIC_BUFFERS                 16
#define NUM_MIC_BUFFERS_MASK            0xf
#define NUM_MIC_BYTES_PER_SAMPLE        2
// NUM_MIC_SAMPLES_PACKET must be even since it
// counts 16-bit samples and I2S transfers 32-bit
// packed samples
#define NUM_MIC_SAMPLES_PACKET          120

#define MIC_SIM_PERIOD                  1000

#define MIC_DATA_READY                  0x00000001
#define MIC_CNTL_PACKET_RECEIVED        0x00000002
#define MIC_ONOFF_PACKET_RECEIVED       0x00000004
#define MIC_VOLUME_PACKET_RECEIVED      0x00000008
#define MIC_EVENT_MASK                  0x0000000F

#define UUID_DXTER_BASE 0x0C, 0x0B, 0x0A, 0x09, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01
#define UUID_DXTER_STETH_SERVICE  0x00, 0x00, 0x00, 0xAF
#define UUID_DXTER_STETH_MIC_CNTL 0x01, 0x00, 0x00, 0xAF
#define UUID_DXTER_STETH_MIC_VOL  0x02, 0x00, 0x00, 0xAF
#define UUID_DXTER_STETH_MIC_DATA 0x03, 0x00, 0x00, 0xAF

#define UUID_DXTER_STETHOSCOPE_SERVICE       BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_STETH_SERVICE )
#define UUID_DXTER_STETHOSCOPE_MIC_CNTL_CHAR BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_STETH_MIC_CNTL )
#define UUID_DXTER_STETHOSCOPE_MIC_VOL_CHAR  BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_STETH_MIC_VOL )
#define UUID_DXTER_STETHOSCOPE_MIC_DATA_CHAR BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_STETH_MIC_DATA )


// micCntl controls the microphone
typedef struct {
  uint32_t     off0_on1;
} micCtrlCharacteristic;

// Just control the volume
typedef struct {
  uint32_t     volume;
  uint32_t     analog_volume;
} micVolCharacteristic;

// Data from microphone to tablet
// Note: micData must be first since its address
// is used to identify the Characteristic as well
// as identify the actual data storage
typedef struct {
  union {
    int16_t   hw[NUM_MIC_SAMPLES_PACKET];
    uint32_t  lw[NUM_MIC_SAMPLES_PACKET / 2];
  } micData;
  uint16_t     seqNum;
  uint16_t     chkSum;
} micDataCharacteristic;

static micCtrlCharacteristic micCtrl = {
               .off0_on1 = 0,
};

static micVolCharacteristic micVol = {
               .volume = 0,
               .analog_volume = 0,
};

static uint8_t  sgtl5000CmdBuf[4];
static uint16_t sgtl5000InitCntr;
static uint16_t sgtl5000InitDone = 0;

// An array of RegisterAddress - Value pairs used as an initialization sequence
static uint16_t sgtl5000InitSeq[] = {
  // Only needed if external power for VDDD
  SGTL5000_CHIP_ANA_POWER,          DAC_STEREO | ADC_STEREO | REFTOP_POWER_ON,
  //SGTL5000_CHIP_ANA_POWER,          DAC_STEREO | LINREG_D_POWER_ON | ADC_STEREO | REFTOP_POWER_ON,
  // Configure charge pump to use VDDIO
  //SGTL5000_CHIP_LINREG_CTRL,        VDDD_OUT_1_20V | VDDC_ASSN_MANUAL | VDDC_MAN_VDDIO,
  // Reference Voltage and Bias Current Config
  // Set ground, ADC, DAC reference voltage to VDDA/2
  // Since VDDA is 3.3V, VDDA/2 is 1.65V
  // Bias current set to 50% of nominal
  // Set SLOW ramp up rate to minimze pop
  SGTL5000_CHIP_REF_CTRL,           VAG_1_550V | BIAS_M50_0_PERCENT | SMALL_POP_SLOW,
  // LineOut Reference Voltage and Bias Current config
  // Set LINEOUT reference voltage to VDDIO/2
  // Since VDDIO is 3.3V, VDDA/2 is 1.65V
  // Bias current set to 0.36mA
  SGTL5000_CHIP_LINE_OUT_CTRL,      LINE_OUT_VAG_1_550V | LINE_OUT_CURRENT_0_36mA,
  // Enable Short detect for headphone L/R and Center
  // Set short detect current trip level to 75mA
  SGTL5000_CHIP_SHORT_CTRL,         LVLADJL_075mA | LVLADJR_075mA | LVLADJC_200mA
                                                  | MODE_CM_SHORT_DETECT_AUTO | MODE_LR_SHORT_DETECT_ENABLED,
  // Enable Zero-cross detect for HP_OUT and ADC
  SGTL5000_CHIP_ANA_CTRL,           MUTE_LINEOUT_MUTED | MUTE_ADC_MUTED | MUTE_HP_MUTED
                                                       | EN_ZCD_ADC_ENABLED | EN_ZCD_HP_ENABLED,
  // Power up Inputs/Outputs/Digital Blocks
  // Power up LINEOUT, HP, ADC, DAC
  //SGTL5000_CHIP_ANA_POWER,          DAC_STEREO | LINREG_D_POWER_ON | VAG_POWER_ON | ADC_STEREO | REFTOP_POWER_ON
  //                                             | HP_POWER_ON | DAC_POWER_ON | CAPLESS_HP_POWER_ON | ADC_POWER_ON 
  //                                             | LINEOUT_POWER_ON,
  SGTL5000_CHIP_ANA_POWER,          DAC_STEREO | VAG_POWER_ON | ADC_STEREO | REFTOP_POWER_ON
                                               | HP_POWER_ON | DAC_POWER_ON | CAPLESS_HP_POWER_ON | ADC_POWER_ON 
                                               | LINEOUT_POWER_ON,
  // Power up digital blocks
  // I2S_OUT, DAP, ADC, DAC
  SGTL5000_CHIP_DIG_POWER,          ENABLE_I2S_OUT_POWER | ENABLE_DAP_POWER | ENABLE_DAC_POWER | ENABLE_ADC_POWER,
  // Set LINEOUT volume
  // Based on voltage reference (VAG)
  // Value = (int)(40*log(VAG_VAL/LO_VAGCNTRL)+15)
  // See table in spec sheet
  SGTL5000_CHIP_LINE_OUT_VOL,       0x0F0F,
  // Set Clocks
  // Make sure PLL and VCOAMP are powered up
  //SGTL5000_CHIP_ANA_POWER,          DAC_STEREO | LINREG_D_POWER_ON | VAG_POWER_ON | ADC_STEREO | REFTOP_POWER_ON
  //                                             | HP_POWER_ON | DAC_POWER_ON | CAPLESS_HP_POWER_ON | ADC_POWER_ON 
  //                                             | LINEOUT_POWER_ON | VCOAMP_POWER_ON | PLL_POWER_ON,
  SGTL5000_CHIP_ANA_POWER,          DAC_STEREO | VAG_POWER_ON | ADC_STEREO | REFTOP_POWER_ON
                                               | HP_POWER_ON | DAC_POWER_ON | CAPLESS_HP_POWER_ON | ADC_POWER_ON 
                                               | LINEOUT_POWER_ON | VCOAMP_POWER_ON | PLL_POWER_ON,
  // MCLK_IN is 12 MHz
  // We want a 16 KHz sample rate
  // INT divisor is Floor(PLL_OUT / MCLK_IN)
  //        196.608 MHz / 12 MHz = 16.384
  //        INT divisor is 16
  // FRAC divisor is ((PLL_OUT / MCLK_IN) - INTdivisor) * 2048
  //        ((196.608 MHz / 12 MHz) - 16) * 2048
  //        0.384 * 2048 = 768.432
  //        FRAC divisor is 786
  // Yields PLL_OUT of 196.608 MHz
  SGTL5000_CHIP_PLL_CTRL,           (16 << 11) | (786 << 0),
  // Set Clock Control
  // Input is PLL
  // Sample Rate set to 32KHz
  // Set Rate to 1/2 of SYS_FS
  // Should yield Fx of 16 KHz
  SGTL5000_CHIP_CLK_CTRL,           FREQ_PLL | SAMP_32KHZ | RATE_MODE_0_50,
  // Set up I2S to be Master, have 16 bits in I2S format
  // Fx x 32??
  SGTL5000_CHIP_I2S_CTRL,           LRPOL_L0R1 | LRALIGN_DELAY | I2S_MODE_LEFT_JUSTIFIED | I2S_SIZE_16
                                               | I2S_SCLK_RISING | I2S_MASTER | I2S_SCLK_FREQ_FSx32,
  // Set up Microphone
  // Sets Bias, Gain, and Impedance
  SGTL5000_CHIP_MIC_CTRL,           MIC_BIAS_3_00V | MIC_GAIN_40dB | MIC_BIAS_RESISTOR_2KOhm,
  // Set up routing through chip
  // MIC_IN to ADC to I2S_OUT
  // MIC_IN to ADC to DAC to HP_OUT (for HeadPhone)
  // MIC_IN to ADC_to DAC to LINEOUT
  // After ADC, can also send through DAP
  // MIC_IN -> ADC -> I2S_OUT
  // ADC -> HEADPHONE
  SGTL5000_CHIP_ANA_CTRL,           MUTE_LINEOUT_MUTED | MUTE_ADC_UNMUTED | MUTE_HP_UNMUTED
                                                       | EN_ZCD_ADC_ENABLED | EN_ZCD_HP_ENABLED
                                                       | SELECT_ADC_INPUT_MICROPHONE | SELECT_HP_INPUT_DAC,
  SGTL5000_CHIP_SSS_CTRL,           I2S_OUTPUT_ADC | I2S_DAC_INPUT_ADC
};

#if 0
static micDataCharacteristic initMicDataChar = { { {  0,  1,  2,  3,  4,  5,  6,  7,  8,  9,
                                                     10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                                                     20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                                                     30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                                                     40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
                                                     50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
                                                     60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
                                                     70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
                                                     80, 81, 82, 83, 84, 85, 86, 87, 88, 89,
                                                     90, 91, 92, 93, 94, 95, 96, 97, 97, 99,
                                                     100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
                                                     110, 111, 112, 113, 114, 115, 116, 117, 118, 119 } },
                                                     0, 0 };
#endif


#define MIC_EVENT_INITIALIZATION       10
#define MIC_EVENT_CNTL                  1
#define MIC_EVENT_VOL                   2
#define MIC_EVENT_DATA                  3
#define MIC_EVENT_TEMP_AVAILABLE        4
#define MIC_EVENT_TEMP_INIT_DONE        5

struct k_poll_signal micEvent;

static micDataCharacteristic micData[NUM_MIC_BUFFERS];
static bleSendQueueItem      micQueueItem[NUM_MIC_BUFFERS];

static int micCharCntr = 0;          // Will count 0 - 7
static int micCharMask = 0x7;        // 0x7 represents 0-7

static bool micNotifyEnabled= false;

static void mic_ccc_cfg_changed(const struct bt_gatt_attr *attr,
                                uint16_t value) {
	if(BT_GATT_CCC_NOTIFY == value) {
          micNotifyEnabled= true;
        } else {
          micNotifyEnabled= false;
        }
}

static ssize_t micCtrlWritten(struct bt_conn *conn,
                              const struct bt_gatt_attr *attr,
                              const void *buf,
                              uint16_t len,
                              uint16_t offset,
                              uint8_t flags) {
  micCtrlCharacteristic *micCntlBufPtr= (micCtrlCharacteristic *)buf;
  micCtrl.off0_on1= sys_cpu_to_le32(micCntlBufPtr->off0_on1);
  printk("micCtrl %d written    to %d\n",len,micCtrl.off0_on1);
  k_poll_signal_raise(&micEvent,MIC_EVENT_CNTL);
  return 4;
}

static ssize_t micVolWritten(struct bt_conn *conn,
                             const struct bt_gatt_attr *attr,
                             const void *buf,
                             uint16_t len,
                             uint16_t offset,
                             uint8_t flags) {
	printk("micVol written\n");
        return 0;
}

static ssize_t micDataRead(struct bt_conn *conn,
                             const struct bt_gatt_attr *attr,
                             void *buf,
                             uint16_t len,
                             uint16_t offset) {
	printk("micData read\n");
        return 0;
}




BT_GATT_SERVICE_DEFINE(micService,
	BT_GATT_PRIMARY_SERVICE(UUID_DXTER_STETHOSCOPE_SERVICE),     // 0 - Service declaration 

	/* Mic Control */
	BT_GATT_CHARACTERISTIC(UUID_DXTER_STETHOSCOPE_MIC_CNTL_CHAR, // 1 - UUID         -- Value Handle
			         BT_GATT_CHRC_WRITE                  //     Props
                               | BT_GATT_CHRC_WRITE_WITHOUT_RESP,
			         BT_GATT_PERM_READ                   //     Permission
                               | BT_GATT_PERM_WRITE,
			       NULL,                                 //     Read CB
                               micCtrlWritten,                       //     Write CB
                               &micCtrl.off0_on1),                   // 2 - &Value       <- Char Value
	BT_GATT_CUD(           "Mic On/Off Ctrl",                    // 3 - String
                               BT_GATT_PERM_READ),                   //     Permission
	/* Mic Volume */
	BT_GATT_CHARACTERISTIC(UUID_DXTER_STETHOSCOPE_MIC_VOL_CHAR,  // 4 - UUID         -- Value Handle
			       BT_GATT_CHRC_WRITE,                   //     Props
			       BT_GATT_PERM_READ,                    //     Permission
			       NULL,                                 //     Read CB
                               micVolWritten,                        //     Write CB
                               &micVol),                             // 5 - &Value       <- Char Value
	BT_GATT_CUD(           "Mic Vol Ctrl",                       // 6 - String
                               BT_GATT_PERM_READ),                   //     Permission
	/* Mic Data */
	BT_GATT_CHARACTERISTIC(UUID_DXTER_STETHOSCOPE_MIC_DATA_CHAR, // 7 - UUID         -- Value Handle
			         BT_GATT_CHRC_READ                   //     Props
                               | BT_GATT_CHRC_NOTIFY,
			       BT_GATT_PERM_READ,                    //     Permission
			       micDataRead,                          //     Read CB
                               NULL,                                 //     Write CB
                               &(micQueueItem[0].data)),             // 8 - &Value       <- Char Value
	BT_GATT_CCC(           mic_ccc_cfg_changed,                  // 9 - Write CB
		                 BT_GATT_PERM_READ                   //     Permission
                               | BT_GATT_PERM_WRITE),
	BT_GATT_CUD(           "Mic Data",                           // 10- String
                               BT_GATT_PERM_READ),                   //     Permission

);


void
micSimTimerExpire(struct k_timer *t) {
//  printk("Send a packet\n");
  k_poll_signal_raise(&micEvent,MIC_EVENT_DATA);
}

K_TIMER_DEFINE(micSimTimer,micSimTimerExpire,NULL);


void
i2sHandler(nrfx_i2s_buffers_t *ptrReleased, uint32_t status) {

}


void
twimMicCallback(nrfx_twim_evt_t const *evtPtr, void *jobPtr) {
  TWIM_QUEUE_ITEM *currJob= (TWIM_QUEUE_ITEM *)jobPtr;

  if(evtPtr->type == NRFX_TWIM_EVT_DONE) {
//    printk("TWIM transfer finished. jobPtr: %d\n",*jobContext);
    switch(currJob->jobContext) {
      case MIC_EVENT_TEMP_AVAILABLE: {
        k_poll_signal_raise(&micEvent,MIC_EVENT_TEMP_AVAILABLE);
      }
      break;
      case MIC_EVENT_INITIALIZATION:
        k_poll_signal_raise(&micEvent,MIC_EVENT_INITIALIZATION);
      break;

      case MIC_EVENT_CNTL:
      break;

      case MIC_EVENT_TEMP_INIT_DONE:
      break;
    }
  } else {
    printk("*******twimMicCallback, but not Done! Type: %d\n",evtPtr->type);
  }
}



micDataCharacteristic testChar;
int16_t dummyData= 0;


  // Initialize Temp chip
#define LM75A_ADDR       0x48
#define LM75A_REG_CONF   0x01
#define LM75A_REG_TEMP   0x00
static uint8_t initWrite[] = { LM75A_REG_CONF, 0x00 };
static uint8_t readWrite[] = { LM75A_REG_TEMP };
static uint8_t readBuf[2];


TWIM_QUEUE_ITEM lm75aInit = {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TX,
  .xfer_desc.address          = LM75A_ADDR,
  .xfer_desc.primary_length   = sizeof(initWrite),
  .xfer_desc.secondary_length = 0,
  .xfer_desc.p_primary_buf    = initWrite,
  .xfer_desc.p_secondary_buf  = NULL,
  .flags                      = 0,
  .twim_cb                    = NULL,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = MIC_EVENT_TEMP_INIT_DONE
};

TWIM_QUEUE_ITEM lm75aMicRead = {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TXRX,
  .xfer_desc.address          = LM75A_ADDR,
  .xfer_desc.primary_length   = sizeof(readWrite),
  .xfer_desc.secondary_length = 2,
  .xfer_desc.p_primary_buf    = readWrite,
  .xfer_desc.p_secondary_buf  = readBuf,
  .flags                      = 0,
  .twim_cb                    = twimMicCallback,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = MIC_EVENT_TEMP_AVAILABLE
};

TWIM_QUEUE_ITEM sgtl5000Init = {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TX,
  .xfer_desc.address          = SGTL5000_I2C_ADDRESS,
  .xfer_desc.primary_length   = 4,
  .xfer_desc.secondary_length = 0,
  .xfer_desc.p_primary_buf    = &sgtl5000CmdBuf[0],
  .xfer_desc.p_secondary_buf  = NULL,
  .flags                      = 0,
  .twim_cb                    = twimMicCallback,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = MIC_EVENT_INITIALIZATION
};


void
threadMicrophone(void *i2cInstInfoPtr, void *unused2, void *unused3) {
  uint32_t micOff0On1 = 0;
  int i;
  uint32_t seqNum= 0;
  uint32_t chkSum= 0;
  int err= NRFX_SUCCESS;
  TWIM_MGR_INFO *i2cInfoPtr= (TWIM_MGR_INFO *)i2cInstInfoPtr;


#if 0
  // Initialize the I2S used here
  // I2S configure
  nrfx_i2s_config_t i2sConfig = {
    .sck_pin      = I2S_SCK_PIN,
    .lrck_pin     = I2S_LRCK_PIN,
    .mck_pin      = I2S_MCK_PIN,
    .sdout_pin    = NRFX_I2S_PIN_NOT_USED,
    .sdin_pin     = I2S_SDIN_PIN,
    .irq_priority = NRFX_I2S_DEFAULT_CONFIG_IRQ_PRIORITY,
    .mode         = NRF_I2S_MODE_MASTER,
    .format       = NRF_I2S_FORMAT_I2S,
    .alignment    = NRF_I2S_ALIGN_LEFT,
    .sample_width = NRF_I2S_SWIDTH_16BIT,
    .channels     = NRF_I2S_CHANNELS_LEFT,
    .mck_setup    = NRF_I2S_MCK_32MDIV8,
    .ratio        = NRF_I2S_RATIO_32X
  };
  err= nrfx_i2s_init(&i2sConfig,
                     (nrfx_i2s_data_handler_t)i2sHandler
                    );
  if(NRFX_SUCCESS != err) {
    printk("I2S driver init failed: %d\n", err);
  }
#endif

  // Create a signal used to indicate when a packet should be sent
  k_poll_signal_init(&micEvent);
  // Use that signal to create an event condition
  struct k_poll_event micEvents[1] = {
    K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL,
                             K_POLL_MODE_NOTIFY_ONLY,
                             &micEvent),
  };

#if 0
  // Initialize Temp chip
  #define LM75A_ADDR       0x48
  #define LM75A_REG_CONF   0x01
  #define LM75A_REG_TEMP   0x00
//  uint8_t initWrite[] = { LM75A_REG_CONF, 0x00 };
  static uint8_t readWrite[] = { LM75A_REG_TEMP };
#endif

  lm75aInit.twimMgrInfoPtr      = i2cInfoPtr;
  lm75aMicRead.twimMgrInfoPtr   = i2cInfoPtr;
  sgtl5000Init.twimMgrInfoPtr   = i2cInfoPtr;

#if 1
  // Initialize the LM75A chip
  err= twimScheduleJob(&lm75aInit);
  if(NRFX_SUCCESS != err) {
    printk("TWIM error during init!\n");
  }
#endif



  // Thread body
  for(;;) {
    // Block until an event is received
    k_poll(micEvents, 1, K_FOREVER);
    // An event has been received
    // Clear the event flags/state
    micEvents[0].signal->signaled= 0;
    micEvents[0].state= K_POLL_STATE_NOT_READY;

    // Check if event is for Data Transfer
    if(    (micEvents[0].signal->result == MIC_EVENT_DATA)
        && (micNotifyEnabled == true)
        && (micOff0On1 == 1) ) {
      // Send data 
#if 0
      // Create dummy data for now
      for(i= 0; i < NUM_MIC_SAMPLES_PACKET; i++) {
        micData[micCharCntr].micData.hw[i]= dummyData++;
      }
//printk("Generated packet: %d\n",seqNum);
      // Calculate the check sum over the data
      uint16_t *sampPtr= &(testChar.micData.hw[0]);
      for(i= 0, chkSum= 0; i < NUM_MIC_SAMPLES_PACKET; i++) {
        chkSum= chkSum + (uint32_t)((*sampPtr++) & 0xffff);
      }
      // Add in the Sequence Number, then increment it
      chkSum= chkSum + seqNum;
      // Put sequence number and checksum into the structure
      micData[micCharCntr].seqNum= seqNum++;
      micData[micCharCntr].chkSum= chkSum;
      // Populate the queue item
      micQueueItem[micCharCntr].fifo_reserved  = NULL,
      micQueueItem[micCharCntr].conn           = bleCurrentConnection;
      micQueueItem[micCharCntr].characteristic = (struct bt_gatt_attr *)&micService.attrs[8],
      micQueueItem[micCharCntr].len            = sizeof(micDataCharacteristic);
      micQueueItem[micCharCntr].data           = (void *)&micData[micCharCntr];
      // Send it via the FIFO to the send thread
//printk("     Sending via FIFO\n");
      k_fifo_put(&notifyQueue,&micQueueItem[micCharCntr]);
      // Update micCharCntr
      micCharCntr= (micCharCntr + 1) & micCharMask;
#endif
printk("Start micTemp\n");
#if 1
      err= twimScheduleJob(&lm75aMicRead);
      if(NRFX_SUCCESS != err) {
        printk("TWIM error during read!\n");
      }
#endif
    // Check if event is for Control
    } else if(micEvents[0].signal->result == MIC_EVENT_CNTL) {
      if(    (micOff0On1 == 0)
          && (micCtrl.off0_on1 == 1)) {
        // Mic is off, turn it on
printk("Turn Mic on\n");
        // Initialize the audio chip; start the first xfer; others will follow...
        sgtl5000InitCntr= 0;
        sgtl5000InitDone= 0;
        uint16ToBytes(sgtl5000InitSeq[sgtl5000InitCntr],sgtl5000CmdBuf,0);
        sgtl5000InitCntr++;
        uint16ToBytes(sgtl5000InitSeq[sgtl5000InitCntr],sgtl5000CmdBuf,2);
        sgtl5000InitCntr++;
        err= twimScheduleJob(&sgtl5000Init);
        if(NRFX_SUCCESS != err) {
          printk("TWIM error during audio init!\n");
        }

        k_timer_start(&micSimTimer,K_MSEC(3000),K_MSEC(MIC_SIM_PERIOD));
        seqNum= 0;
        micCharCntr= 0;
        micOff0On1= 1;
      } else if(    (micOff0On1 == 1)
                 && (micCtrl.off0_on1 == 0)) {
        // Mic is on, turn it off
printk("Turn Mic off\n");
        k_timer_stop(&micSimTimer);
        micOff0On1= 0;
      }
    } else if(micEvents[0].signal->result == MIC_EVENT_INITIALIZATION) {
      // Check if initialization is done; if not start the next one
      if(sgtl5000InitCntr < sizeof(sgtl5000InitSeq)/2) {
        // Haven't gone through the whole table yet
        uint16ToBytes(sgtl5000InitSeq[sgtl5000InitCntr],sgtl5000CmdBuf,0);
        sgtl5000InitCntr++;
        uint16ToBytes(sgtl5000InitSeq[sgtl5000InitCntr],sgtl5000CmdBuf,2);
        sgtl5000InitCntr++;
        err= twimScheduleJob(&sgtl5000Init);
        if(NRFX_SUCCESS != err) {
          printk("TWIM error during audio init!\n");
        }
      } else {
        // Initialization sequence is done
        sgtl5000InitDone= 1;
      }
    } else if(micEvents[0].signal->result == MIC_EVENT_TEMP_AVAILABLE) {
      int16_t tempCRaw = ((readBuf[0] << 8) | readBuf[1]) / 32;
      float tempC = (float)tempCRaw * 0.125;
      float tempF = (tempC * 1.8) + 32;
      char strBuf[128];
      sprintf(strBuf,"MicTemp F: %f   Temp C: %f\n",tempF,tempC);
printk("%s\n",strBuf);
    }
  }
}

