#include "TeleMod.h"

// Define the Temperature Sensor characteristic
//
#define NUM_TEMP_BUFFERS                  4
#define NUM_TEMP_BUFFERS_MASK             0x3

#define MLX90615_TURN_ON_DELAY_MS         500
#define MLX90615_WAKE_DELAY_MS            300

#define MLX90615_SCALE                    0.02
#define MLX90615_OFFSET                   273.15

#define MLX90615_I2C_ADDRESS              0x5B
#define I2C_READ                          0x01
#define I2C_WRITE                         0x00

#define MLX90615_ENTER_SLEEP              0xC6
#define MLX90615_RAM_ACCESS               0x20

#define MLX90615_SLEEP_PEC                0x6D
#define MLX90615_REG_TA_DATA              0x06
#define MLX90615_REG_TO_DATA              0x07



/*******************************************************************************
 * @fn          Mlx90615PecCheck
 *
 * @brief       Check the packet data against the PEC
 *
 * @param       Array of bytes to check
 *
 * @param       Number of bytes to check
 *
 *              The CRC used is: X8+X2+X1+1
 *
 * @return      true if passed
 ******************************************************************************/

static const uint8_t crc_table[] = {
    0x00, 0x07, 0x0e, 0x09, 0x1c, 0x1b, 0x12, 0x15, 0x38, 0x3f, 0x36, 0x31,
    0x24, 0x23, 0x2a, 0x2d, 0x70, 0x77, 0x7e, 0x79, 0x6c, 0x6b, 0x62, 0x65,
    0x48, 0x4f, 0x46, 0x41, 0x54, 0x53, 0x5a, 0x5d, 0xe0, 0xe7, 0xee, 0xe9,
    0xfc, 0xfb, 0xf2, 0xf5, 0xd8, 0xdf, 0xd6, 0xd1, 0xc4, 0xc3, 0xca, 0xcd,
    0x90, 0x97, 0x9e, 0x99, 0x8c, 0x8b, 0x82, 0x85, 0xa8, 0xaf, 0xa6, 0xa1,
    0xb4, 0xb3, 0xba, 0xbd, 0xc7, 0xc0, 0xc9, 0xce, 0xdb, 0xdc, 0xd5, 0xd2,
    0xff, 0xf8, 0xf1, 0xf6, 0xe3, 0xe4, 0xed, 0xea, 0xb7, 0xb0, 0xb9, 0xbe,
    0xab, 0xac, 0xa5, 0xa2, 0x8f, 0x88, 0x81, 0x86, 0x93, 0x94, 0x9d, 0x9a,
    0x27, 0x20, 0x29, 0x2e, 0x3b, 0x3c, 0x35, 0x32, 0x1f, 0x18, 0x11, 0x16,
    0x03, 0x04, 0x0d, 0x0a, 0x57, 0x50, 0x59, 0x5e, 0x4b, 0x4c, 0x45, 0x42,
    0x6f, 0x68, 0x61, 0x66, 0x73, 0x74, 0x7d, 0x7a, 0x89, 0x8e, 0x87, 0x80,
    0x95, 0x92, 0x9b, 0x9c, 0xb1, 0xb6, 0xbf, 0xb8, 0xad, 0xaa, 0xa3, 0xa4,
    0xf9, 0xfe, 0xf7, 0xf0, 0xe5, 0xe2, 0xeb, 0xec, 0xc1, 0xc6, 0xcf, 0xc8,
    0xdd, 0xda, 0xd3, 0xd4, 0x69, 0x6e, 0x67, 0x60, 0x75, 0x72, 0x7b, 0x7c,
    0x51, 0x56, 0x5f, 0x58, 0x4d, 0x4a, 0x43, 0x44, 0x19, 0x1e, 0x17, 0x10,
    0x05, 0x02, 0x0b, 0x0c, 0x21, 0x26, 0x2f, 0x28, 0x3d, 0x3a, 0x33, 0x34,
    0x4e, 0x49, 0x40, 0x47, 0x52, 0x55, 0x5c, 0x5b, 0x76, 0x71, 0x78, 0x7f,
    0x6a, 0x6d, 0x64, 0x63, 0x3e, 0x39, 0x30, 0x37, 0x22, 0x25, 0x2c, 0x2b,
    0x06, 0x01, 0x08, 0x0f, 0x1a, 0x1d, 0x14, 0x13, 0xae, 0xa9, 0xa0, 0xa7,
    0xb2, 0xb5, 0xbc, 0xbb, 0x96, 0x91, 0x98, 0x9f, 0x8a, 0x8d, 0x84, 0x83,
    0xde, 0xd9, 0xd0, 0xd7, 0xc2, 0xc5, 0xcc, 0xcb, 0xe6, 0xe1, 0xe8, 0xef,
    0xfa, 0xfd, 0xf4, 0xf3
};

uint8_t
Mlx90615PecCheck(uint8_t *chkBytes, uint8_t len) {
  uint16_t i;
  uint16_t crc = 0x0;

  while (len--) {
    i = (crc ^ *chkBytes++) & 0xFF;
    crc = (crc_table[i] ^ (crc << 8)) & 0xFF;
  }

  return crc & 0xFF;
}

#define UUID_DXTER_BASE 0x0C, 0x0B, 0x0A, 0x09, 0x08, 0x07, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01
#define UUID_DXTER_TEMP_SERVICE     0x00, 0x00, 0x00, 0xAE
#define UUID_DXTER_TEMP_CNTL        0x01, 0x00, 0x00, 0xAE
#define UUID_DXTER_TEMP_DATA        0x02, 0x00, 0x00, 0xAE

#define UUID_DXTER_TEMPERATURE_SERVICE       BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_TEMP_SERVICE )
#define UUID_DXTER_TEMPERATURE_CNTL_CHAR     BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_TEMP_CNTL )
#define UUID_DXTER_TEMPERATURE_DATA_CHAR     BT_UUID_DECLARE_128( UUID_DXTER_BASE, UUID_DXTER_TEMP_DATA )


// tempCntl controls the Temperature
typedef struct {
  uint32_t     off0_on1;
  uint32_t     sampPeriodMS;
} tempCtrlCharacteristic;

// Data from temperature sensor to tablet
typedef struct {
  uint16_t     tempAmbient;
  uint16_t     tempObjectKelvin;
  float        tempObjectCelsius;
  float        tempObjectFahrenheit;
  char         tempString[128];
} tempDataCharacteristic;
 
static tempCtrlCharacteristic tempCtrl = {
               .off0_on1     = 0,
               .sampPeriodMS = 500, 
};

#define TEMP_EVENT_WAKE_UP_PULSE  10
#define TEMP_EVENT_ENTER_SLEEP    11
#define TEMP_EVENT_CNTL_RECEIVED  12
#define TEMP_EVENT_SAMPLE_TIME    13
#define TEMP_EVENT_DATA_TA_AVAIL  14
#define TEMP_EVENT_DATA_TO_AVAIL  15

struct k_poll_signal tempEvent;

static tempDataCharacteristic tempData[NUM_TEMP_BUFFERS];
static bleSendQueueItem       tempQueueItem[NUM_TEMP_BUFFERS];

static int tempCharCntr = 0;         // Will count 0 - 3, then repeat

static bool tempNotifyEnabled= false;

static void temp_ccc_cfg_changed(const struct bt_gatt_attr *attr,
                                 uint16_t value) {
  if(BT_GATT_CCC_NOTIFY == value) {
    tempNotifyEnabled= true;
  } else {
    tempNotifyEnabled= false;
  }
}

static ssize_t tempCtrlWritten(struct bt_conn *conn,
                               const struct bt_gatt_attr *attr,
                               const void *buf,
                               uint16_t len,
                               uint16_t offset,
                               uint8_t flags) {
  tempCtrlCharacteristic *tempCntlBufPtr= (tempCtrlCharacteristic *)buf;
  tempCtrl.off0_on1=      sys_cpu_to_le32(tempCntlBufPtr->off0_on1);
  tempCtrl.sampPeriodMS=  sys_cpu_to_le32(tempCntlBufPtr->sampPeriodMS);
printk("New temp Cntl: on/off: %d   period: %d\n",tempCtrl.off0_on1,tempCtrl.sampPeriodMS);
  k_poll_signal_raise(&tempEvent,TEMP_EVENT_CNTL_RECEIVED);
  return sizeof(tempCtrlCharacteristic);
}

static ssize_t tempDataRead(struct bt_conn *conn,
                            const struct bt_gatt_attr *attr,
                            void *buf,
                            uint16_t len,
                            uint16_t offset) {
  printk("tempData read\n");
  return 0;
}

BT_GATT_SERVICE_DEFINE(tempService,
	BT_GATT_PRIMARY_SERVICE(UUID_DXTER_TEMPERATURE_SERVICE),     // 0 - Service declaration 

	/* Temp Control */
	BT_GATT_CHARACTERISTIC(UUID_DXTER_TEMPERATURE_CNTL_CHAR,     // 1 - UUID         -- Value Handle
			         BT_GATT_CHRC_WRITE                  //     Props
                               | BT_GATT_CHRC_WRITE_WITHOUT_RESP,
			         BT_GATT_PERM_READ                   //     Permission
                               | BT_GATT_PERM_WRITE,
			       NULL,                                 //     Read CB
                               tempCtrlWritten,                      //     Write CB
                               &tempCtrl.off0_on1),                  // 2 - &Value       <- Char Value
	BT_GATT_CUD(           "Temp On/Off Ctrl",                   // 3 - String
                               BT_GATT_PERM_READ),                   //     Permission
	/* Temp Data */
	BT_GATT_CHARACTERISTIC(UUID_DXTER_TEMPERATURE_DATA_CHAR,     // 4 - UUID         -- Value Handle
			         BT_GATT_CHRC_READ                   //     Props
                               | BT_GATT_CHRC_NOTIFY,
			       BT_GATT_PERM_READ,                    //     Permission
			       tempDataRead,                         //     Read CB
                               NULL,                                 //     Write CB
                               &(tempQueueItem[0].data)),            // 5 - &Value       <- Char Value
	BT_GATT_CCC(           temp_ccc_cfg_changed,                 // 6 - Write CB
		                 BT_GATT_PERM_READ                   //     Permission
                               | BT_GATT_PERM_WRITE),
	BT_GATT_CUD(           "Temp Data",                          // 7 - String
                               BT_GATT_PERM_READ),                   //     Permission

);

void
tempSampleTimerExpire(struct k_timer *t) {
  // Timer expired, so get a sample
  k_poll_signal_raise(&tempEvent,TEMP_EVENT_SAMPLE_TIME);
}

K_TIMER_DEFINE(tempSampleTimer,tempSampleTimerExpire,NULL);

// Callback for when I2C has completed
void
twimTempCallback(nrfx_twim_evt_t const *evtPtr, void *jobPtr) {
  if(evtPtr->type == NRFX_TWIM_EVT_DONE) {
    TWIM_QUEUE_ITEM *currJob= (TWIM_QUEUE_ITEM *)jobPtr;
    switch(currJob->jobContext) {

      case TEMP_EVENT_DATA_TA_AVAIL:
        // Data from temperature sensor has been read.  It's in readTaCmd[3:4]
        k_poll_signal_raise(&tempEvent,TEMP_EVENT_DATA_TA_AVAIL);
      break;
   
      case TEMP_EVENT_DATA_TO_AVAIL:
        // Data from temperature sensor has been read.  It's in readToCmd[3:4]
        k_poll_signal_raise(&tempEvent,TEMP_EVENT_DATA_TO_AVAIL);
      break;

      case TEMP_EVENT_WAKE_UP_PULSE:
        // Wake up pulse has been issued
        k_poll_signal_raise(&tempEvent,TEMP_EVENT_WAKE_UP_PULSE);
      break;

      case TEMP_EVENT_ENTER_SLEEP:
        // Enter sleep command has been issued
        k_poll_signal_raise(&tempEvent,TEMP_EVENT_ENTER_SLEEP);
      break;
    }
  }
}

static uint8_t sleepWrite[] = {  MLX90615_ENTER_SLEEP, MLX90615_SLEEP_PEC };
static uint8_t readTaCmd[]  = {  ((MLX90615_I2C_ADDRESS << 1) | I2C_WRITE),
                                 MLX90615_RAM_ACCESS | MLX90615_REG_TA_DATA, 
                                 ((MLX90615_I2C_ADDRESS << 1) | I2C_READ),
                                 0x00,
                                 0x00,
                                 0x00
                              };
static uint8_t readToCmd[]  = {  ((MLX90615_I2C_ADDRESS << 1) | I2C_WRITE),
                                 MLX90615_RAM_ACCESS | MLX90615_REG_TO_DATA, 
                                 ((MLX90615_I2C_ADDRESS << 1) | I2C_READ),
                                 0x00,
                                 0x00,
                                 0x00
                              };

// Job structure to put MLX90615 to sleep
TWIM_QUEUE_ITEM mlx90615Sleep = {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TX,
  .xfer_desc.address          = MLX90615_I2C_ADDRESS,
  .xfer_desc.primary_length   = sizeof(sleepWrite),
  .xfer_desc.secondary_length = 0,
  .xfer_desc.p_primary_buf    = sleepWrite,
  .xfer_desc.p_secondary_buf  = NULL,
  .flags                      = 0,
  .twim_cb                    = twimTempCallback,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = TEMP_EVENT_ENTER_SLEEP
};

// Job structure to wake MLX90615
TWIM_QUEUE_ITEM mlx90615Wake = {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TX,
  .xfer_desc.address          = MLX90615_I2C_ADDRESS,
  .xfer_desc.primary_length   = 0,
  .xfer_desc.secondary_length = 0,
  .xfer_desc.p_primary_buf    = NULL,
  .xfer_desc.p_secondary_buf  = NULL,
  .flags                      = TWIM_SPECIAL_SCLK_LOW(50),
  .twim_cb                    = twimTempCallback,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = TEMP_EVENT_WAKE_UP_PULSE
};

// Job structure for MLX90615 Ambient Read
TWIM_QUEUE_ITEM mlx90615ReadTa= {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TXRX,
  .xfer_desc.address          = MLX90615_I2C_ADDRESS,
  .xfer_desc.primary_length   = 1,
  .xfer_desc.secondary_length = 3,
  .xfer_desc.p_primary_buf    = &readTaCmd[1],
  .xfer_desc.p_secondary_buf  = &readTaCmd[3],
  .flags                      = 0,
  .twim_cb                    = twimTempCallback,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = TEMP_EVENT_DATA_TA_AVAIL
};

// Job structure for MLX90615 Object Read
TWIM_QUEUE_ITEM mlx90615ReadTo= {
  //.twimMgrInfoPtr             = i2cInfoPtr,
  .xfer_desc.type             = NRFX_TWIM_XFER_TXRX,
  .xfer_desc.address          = MLX90615_I2C_ADDRESS,
  .xfer_desc.primary_length   = 1,
  .xfer_desc.secondary_length = 3,
  .xfer_desc.p_primary_buf    = &readToCmd[1],
  .xfer_desc.p_secondary_buf  = &readToCmd[3],
  .flags                      = 0,
  .twim_cb                    = twimTempCallback,
  .speed                      = NRF_TWIM_FREQ_400K,
  .jobContext                 = TEMP_EVENT_DATA_TO_AVAIL
};

// The thread that controls the temperature sensor
void
threadThermometer(void *i2cInstInfoPtr, void *unused2, void *unused3) {
  uint32_t tempOff0On1= 0;
  uint32_t tempPeriod=  0;
  int i;
  uint32_t seqNum= 0;
  uint32_t chkSum= 0;
  int err= NRFX_SUCCESS;
  // Pointer to the info structure for I2C/TWIM device being used
  // Convert the pointer to the correct "type"
  TWIM_MGR_INFO *i2cInfoPtr= (TWIM_MGR_INFO *)i2cInstInfoPtr;

  // Create a signal used to indicate when some action should be taken
  k_poll_signal_init(&tempEvent);
  // Use that signal to create an event condition
  struct k_poll_event tempEventConditions[1] = {
    K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL,
                             K_POLL_MODE_NOTIFY_ONLY,
                             &tempEvent),
  };

  // Set values not known until thread starts
  mlx90615Wake.twimMgrInfoPtr      = i2cInfoPtr;
  mlx90615Sleep.twimMgrInfoPtr     = i2cInfoPtr;
  mlx90615ReadTa.twimMgrInfoPtr    = i2cInfoPtr;
  mlx90615ReadTo.twimMgrInfoPtr    = i2cInfoPtr;

  // Assume power just provided to sensor...   Have to wait 
  k_sleep(K_MSEC(MLX90615_TURN_ON_DELAY_MS));

  // Thread body
  for(;;) {
    // Block until an event is received
    k_poll(tempEventConditions, 1, K_FOREVER);

    // An event has been received
    // Clear the event flags/state
    // This is done here since some operations create a new event
    // and if this is put at the bottom, the event is cleared before
    // it can be dealt with
    tempEventConditions[0].signal->signaled= 0;
    tempEventConditions[0].state= K_POLL_STATE_NOT_READY;
    // Check if event is to start a temperature sample
    if(    (tempEventConditions[0].signal->result == TEMP_EVENT_SAMPLE_TIME)
        && (tempNotifyEnabled == true)
        && (tempOff0On1 == 1) ) {
      // Sample the temperature sensor 
      // I2C used to read Ambient and Object temperatures
      err= twimScheduleJob(&mlx90615ReadTa);
      if(NRFX_SUCCESS != err) {
        printk("TWIM error during temp read TA!\n");
      }
      err= twimScheduleJob(&mlx90615ReadTo);
      if(NRFX_SUCCESS != err) {
        printk("TWIM error during temp read TO!\n");
      }

    // Check if event is for Control
    } else if(tempEventConditions[0].signal->result == TEMP_EVENT_CNTL_RECEIVED) {
      if(    (tempOff0On1 == 0)
          && (tempCtrl.off0_on1 == 1)) {
        // Temp is off, turn it on
        seqNum= 0;
        tempCharCntr= 0;
        tempOff0On1= 1;
        tempPeriod= tempCtrl.sampPeriodMS;
        // Wake up the temp sensor
        // Send I2C command to "Wake" the temperature sensor
//printk("Temperature wake\n");
        err= twimScheduleJob(&mlx90615Wake);
        if(NRFX_SUCCESS != err) {
          printk("TWIM error during wake!\n");
        }
      } else if(    (tempOff0On1 == 1)
                 && (tempCtrl.off0_on1 == 0)) {
        // Temp is on, turn it off by stopping the timer
//printk("Temperature turn off\n");
        k_timer_stop(&tempSampleTimer);
        tempOff0On1= 0;
        // Now tell the sensor to go to sleep
        err= twimScheduleJob(&mlx90615Sleep);
        if(NRFX_SUCCESS != err) {
          printk("TWIM error putting temp to sleep!\n");
        }
      } else if(    (tempOff0On1 == 1)
                 && (tempCtrl.off0_on1 == 1)) {
        // Temp is on, new on cmd came in;  just set new rate
        tempPeriod= tempCtrl.sampPeriodMS;
        k_timer_stop(&tempSampleTimer);
        k_timer_start(&tempSampleTimer,K_MSEC(1),K_MSEC(tempPeriod));
      }

    // Check if event is the wake-up pulse has completed
    } else if(tempEventConditions[0].signal->result == TEMP_EVENT_WAKE_UP_PULSE) {
      // The wake-up pulse has been issued; now have to delay for the appropriate time
      // Sensor requires minimum of 300 ms after wake...
      k_sleep(K_MSEC(MLX90615_WAKE_DELAY_MS - 1));
      // After delay, turn on the sample timer
      // Add in the 1 ms taken away above, then start the timer using given period
tempPeriod= 1000;
//printk("Temperature wakeup done\n");
      k_timer_start(&tempSampleTimer,K_MSEC(1),K_MSEC(tempPeriod));

    // Check if sensor was just put to sleep
    } else if(tempEventConditions[0].signal->result == TEMP_EVENT_ENTER_SLEEP) {
      // The "enter sleep" command has finished...   Nothing more to do...

    // Check if TA data is now available
    } else if(tempEventConditions[0].signal->result == TEMP_EVENT_DATA_TA_AVAIL) {
      // Data transfer completed; process the data and send it
      // readTaCmd has the original command plus received data
      uint16_t tempAmbient = (readTaCmd[4] << 8) | readTaCmd[3];
#if 0
      // Now check the result using PEC
for(int ii= 0; ii < 6; ii++) printk("ii: %d  val: %x\n",ii,readTaCmd[ii]);
      uint8_t crcTa= Mlx90615PecCheck(&readTaCmd[0],5);
      if(crcTa != readTaCmd[5]) {
        printk("Ambient Temp CRC failed; is: %x expected: %x\n",crcTa,readTaCmd[5]);
      }
#endif
      tempData[tempCharCntr].tempAmbient=           tempAmbient;

    // Check if TO data is now available
    } else if(tempEventConditions[0].signal->result == TEMP_EVENT_DATA_TO_AVAIL) {
      // Data transfer completed; process the data and send it
      // readToCmd has the original command plus received data
      uint16_t tempObject = (readToCmd[4] << 8) | readToCmd[3];
#if 0
      // Now check the result using PEC
      if(Mlx90615PecCheck(&readToCmd[0],5) != readToCmd[5]) {
        printk("Object Temp CRC failed\n");
      }
#endif
      // Put data into appropriate characteristic (rotate through 4)
      tempData[tempCharCntr].tempObjectKelvin=      tempObject;
      // Convert the raw (Kelvin) temp to F and C
      float t;
      t = ((float)(tempObject)) * MLX90615_SCALE;
      tempData[tempCharCntr].tempObjectCelsius=     t - MLX90615_OFFSET;
      tempData[tempCharCntr].tempObjectFahrenheit=  ((t - MLX90615_OFFSET) * 1.8) + 32;

      sprintf(tempData[tempCharCntr].tempString,"Temp F: %5.2f   Temp C: %5.2f\n",
                                          tempData[tempCharCntr].tempObjectFahrenheit,
                                          tempData[tempCharCntr].tempObjectCelsius);
printk("%s",tempData[tempCharCntr].tempString);
      // Populate the queue item
      tempQueueItem[tempCharCntr].fifo_reserved     = NULL;
      tempQueueItem[tempCharCntr].conn              = bleCurrentConnection;
      tempQueueItem[tempCharCntr].characteristic    = (struct bt_gatt_attr *)&tempService.attrs[5];
      tempQueueItem[tempCharCntr].len               = sizeof(tempDataCharacteristic);
      tempQueueItem[tempCharCntr].data              = (void *)&tempData[tempCharCntr];
      // Send the queue item via the FIFO to the send thread
      k_fifo_put(&notifyQueue,&tempQueueItem[tempCharCntr]);
      // Update tempCharCntr
      tempCharCntr= (tempCharCntr + 1) & NUM_TEMP_BUFFERS_MASK;
    }
  }
}